// ============================================================== 
//                 ORBITER MODULE: LER2
//                  Part of the ORBITER SDK
//          Copyright (C) 2002-2004 Martin Schweiger
//                   All rights reserved
//
// LER2.cpp
// Control module for LER2 vessel class
//
// ============================================================== 

#define STRICT
#define ORBITER_MODULE
#define ENG		1
#define START	2
#include "orbitersdk.h"
#include "LER2.h"
#include "LERMESH.h"
#include "OrbiterSoundSDK35.h"

VISHANDLE MainExternalMeshVisual = 0;

void LER2::SetUMMUAirlockPos (void) {
	int AirlockStatus = Crew.GetAirlockDoorState ();
	Crew.SetActiveDockForTransfer (iActiveDockNumber);
	switch (iActiveDockNumber) {
	case 0:	
		Crew.DefineAirLockShape (AirlockStatus, -5, -2, -4, 2, 0, 2);	
		Crew.SetMembersPosRotOnEVA (_V (-2.268, -1, .421), _V (0, 1.5, 0));
		doorselected = 0;
		break;
	case 1:	
		Crew.DefineAirLockShape (AirlockStatus, -2, 2, -12, 2, -5, -2);	
		Crew.SetMembersPosRotOnEVA (_V (-.548, -1, -3.872), _V (0, 0, 0));
		doorselected = 1;
		break;
	case 2:	
		Crew.DefineAirLockShape (AirlockStatus, -2, 2, -12, 2, -5, -2);	
		Crew.SetMembersPosRotOnEVA (_V (.548, -1, -3.872), _V (0, 0, 0));
		doorselected = 2;
		break;
	}
}

void LER2::clbkPostCreation (void) {
	JohnSoundID = ConnectToOrbiterSoundDLL3 (GetHandle ());
	SoundOptionOnOff3 (JohnSoundID, PLAYMAINTHRUST, FALSE);
	SoundOptionOnOff3 (JohnSoundID, PLAYHOVERTHRUST, FALSE);
	SoundOptionOnOff3 (JohnSoundID, PLAYATTITUDETHRUST, FALSE);
	SoundOptionOnOff3 (JohnSoundID, PLAYCOUNTDOWNWHENTAKEOFF, FALSE);
	SoundOptionOnOff3 (JohnSoundID, PLAYCABINAIRCONDITIONING, FALSE);
	SoundOptionOnOff3 (JohnSoundID, PLAYCABINRANDOMAMBIANCE, FALSE);
	SoundOptionOnOff3 (JohnSoundID, PLAYRADIOATC, FALSE);
	SoundOptionOnOff3 (JohnSoundID, DISPLAYTIMER, FALSE);
}

void LER2::SelectCockpitView (int CAM) {
	switch (CAM) {
	case 0: //FRONT
		SetCameraDefaultDirection (_V (0, 0, 1));
		SetCameraOffset (_V (0, .7, 3.121));
		break;
	case 1: //LSEAT
		SetCameraDefaultDirection (_V (0, 0, 1));
		SetCameraOffset (_V (-.6, .48, 1.412));
		break;
	case 2: //RSEAT
		SetCameraDefaultDirection (_V (0, 0, 1));
		SetCameraOffset (_V (.6, .48, 1.412));
		break;
	case 3: //BUBBLE
		SetCameraDefaultDirection (_V (0, 0, 1));
		SetCameraOffset (_V (0, -.52, 2.512));
		break;
	case 4: //REAR
		SetCameraDefaultDirection (_V (0, 0, -1));
		SetCameraOffset (_V (0, .27, -1.46));
		break;
	}
}

LER2::LER2 (OBJHANDLE hObj, int fmodel): VESSEL2 (hObj, fmodel) {
	whichap = 1;
	SUNSCREEN_status = DOOR_UP;
	i2 = 0;
	i5 = 1;
	DOOR0_status = DOOR_UP;
	DOOR1_status = DOOR_UP;
	TURN_check = TURN_STOP;
	LIFT_SPEED = .1;
	TURN_proc = 0;
	DOOR1_proc = 0;
	SUNSCREEN_proc = 0;
	wrate = 0;
	DefineAnimations ();
	pos = .5;
	i = 0;
	rot = 0;
}

void LER2::clbkSetClassCaps (FILEHANDLE cfg) {
	// physical specs
	SetSize (6.08);
	SetEmptyMass (MASS);
	SetCW (0.9, 0.9, 2, 1.4);
	SetWingAspect (0.1);
	SetWingEffectiveness (0.1);
	SetCrossSections (_V (12.16, 15.99, 9.39));
	SetRotDrag (_V (0.1, 0.1, 0.1));
	if (GetFlightModel () >= 1) {
		SetPitchMomentScale (1e-4);
		SetBankMomentScale (1e-4);
	}
	SetPMI (_V (3.75, 3.48, 2.23));
	SetTrimScale (0.05);
	SetCameraOffset (_V (0, .7, 3.121));
	SetTouchdownPoints (_V (0, -2.162, -0.05774), _V (0.05, -2.162, -0.02887), _V (-0.05, -2.162, -0.02887));
	SetSurfaceFrictionCoeff (0.01, 0.75);
	CreateControlSurface (AIRCTRL_RUDDER, 5.0, 0.2, _V ( 0, 2, -15), AIRCTRL_AXIS_YPOS, anim_rudder);
	PROPELLANT_HANDLE TANK = CreatePropellantResource (FUELMASS);
	// ***************** thruster definitions *******************
	VECTOR3 dirfw = {0, 0, -1};
	VECTOR3 dirbk = {0, 0, 1};
	VECTOR3 dirlt = {1, 0, 0};
	VECTOR3 dirrt = {-1, 0, 0};
	//main thrust
	th_main[0] = CreateThruster (_V (0, 0, 0), dirbk, MAXMAINTH, TANK, ISP);
	thg_main = CreateThrusterGroup (th_main, 1, THGROUP_MAIN);
	//retro thrust
	th_retro[0] = CreateThruster (_V (0, 0, 0), dirfw, MAXMAINTH, TANK, ISP);
	thg_retro = CreateThrusterGroup (th_retro, 1, THGROUP_RETRO);
	//yaw left thrust
	th_yaw_l[0] = CreateThruster (_V (0, 0, -10), dirlt, MAXRCSTH, TANK, RCSISP);
	th_yaw_l[1] = CreateThruster (_V (0, 0, 10), dirrt, MAXRCSTH, TANK, RCSISP);
	thg_yaw_l = CreateThrusterGroup (th_yaw_l, 2, THGROUP_ATT_YAWLEFT);
	//yaw right thrust
	th_yaw_r[0] = CreateThruster (_V (0, 0, 10), dirlt, MAXRCSTH, TANK, RCSISP);
	th_yaw_r[1] = CreateThruster (_V (0, 0, -10), dirrt, MAXRCSTH, TANK, RCSISP);
	thg_yaw_r = CreateThrusterGroup (th_yaw_r, 2, THGROUP_ATT_YAWRIGHT);
	//ummu
	SelectedUmmuMember = 0;
	clbkSetClassCaps_UMMu ();	
	// visual specs
	iMeshMain = AddMesh ("LER_3a"); 
	SetMeshVisibilityMode (iMeshMain, MESHVIS_ALWAYS);
	iMeshEVA1A = AddMesh (oapiLoadMeshGlobal ("LEREVA1"));
	SetMeshVisibilityMode (iMeshEVA1A, MESHVIS_NEVER);
	iMeshEVA2A = AddMesh (oapiLoadMeshGlobal ("LEREVA2"));
	SetMeshVisibilityMode (iMeshEVA2A, MESHVIS_NEVER);
}

// ======================================================= 
// UMMU Class Caps
// ======================================================= 
void LER2::clbkSetClassCaps_UMMu (void) {
	// InitUmmu
	UmmuSdKInitReturnCode = Crew.InitUmmu (GetHandle ());	
	// GetUserUMmuVersion
	float UMmuVersion = Crew.GetUserUMmuVersion ();
	// DefineAirLockShape
	Crew.DefineAirLockShape (FALSE, -5, -2, -4, 2, 0, 2);	// Airlock open, 2 meter large 4 meter high 6 meter long (vessel local coordinate)
	// SetMembersPositionOrientationOnEVA
	Crew.SetMembersPosRotOnEVA (_V (-2.268, -1, .421), _V (0, 1.5, 0));	// 3 meters in front of ship (z) facing direction of ship (vessel local coordinate)
	//	Crew.DefineAirLockShape (AirlockStatus, 2, 5, -3, 2, -2, 2);	
	//Crew.SetMembersPosRotOnEVA (_V (4, -.71, 0), _V (0, 1.5, 0));
	//maximum seat available in this ship
	Crew.SetMaxSeatAvailableInShip (2);
	//default crew member used when the ship is spawned by Orbiter's scenerio editor. 
	Crew.AddCrewMember ("Paul Roper1", 41, 65, 74, "LER");	// (for name and id a-z A-Z 0-9 characters only)
	Crew.AddCrewMember ("Frank Craig1", 27, 67, 55, "LER");	// (for name and id a-z A-Z 0-9 characters only)
	Crew.AddCrewMember ("Craig Collins1", 25, 70, 45, "LER");	// (for name and id a-z A-Z 0-9 characters only)
	Crew.AddCrewMember ("Doug Jackson1", 45, 70, 45, "LER" );	// (for name and id a-z A-Z 0-9 characters only)
	//	Crew.AddCrewMember ("Keith Ford1", 45, 70, 45, "Sec");	// (for name and id a-z A-Z 0-9 characters only)
	//	Crew.AddCrewMember ("Paul Foster1", 45, 70, 45, "Sci" );	// (for name and id a-z A-Z 0-9 characters only)
	//change mesh
	// see function "SendFocusScreenMessage"
	strcpy (cOurMessage, "");
	//change mesh
	//Crew.SetAlternateMeshToUseForEVASpacesuit ("ummu");
	//if ((Crew.GetCrewTotalNumber () == 1)) SetMeshVisibilityMode ( iMeshEVA1A, MESHVIS_ALWAYS );
	// else SetMeshVisibilityMode ( iMeshEVA1A, MESHVIS_NEVER );
	//if ((Crew.GetCrewTotalNumber () == 2)) SetMeshVisibilityMode ( iMeshEVA2A, MESHVIS_ALWAYS );
	// else SetMeshVisibilityMode ( iMeshEVA2A, MESHVIS_NEVER );
}

void LER2::clbkPostStep (double simt, double simdt, double mjd) {
	//---------------------------------------------------------------------------
	// ProcessUniversalMMu
	int ReturnCode = Crew.ProcessUniversalMMu ();
	switch (ReturnCode) {
	case UMMU_TRANSFERED_TO_OUR_SHIP: 
		sprintf (SendFocusScreenMessage (), "%s \"%s\" aged %i was transfered to our ship", Crew.GetCrewMiscIdByName (Crew.GetLastEnteredCrewName ()), Crew.GetLastEnteredCrewName (), Crew.GetCrewAgeByName (Crew.GetLastEnteredCrewName ()));
		break;
	case UMMU_RETURNED_TO_OUR_SHIP:
		sprintf (SendFocusScreenMessage (), "%s \"%s\" aged %i entered into our ship", Crew.GetCrewMiscIdByName (Crew.GetLastEnteredCrewName ()), Crew.GetLastEnteredCrewName (), Crew.GetCrewAgeByName (Crew.GetLastEnteredCrewName ()));
		break;
	}
	//----------------------------------------------------------------------------
	// SetCrewMemberPulseBySlotNumber
	if (GroundContact () == TRUE) {
		// we check vertical speed
		//int I;
		VECTOR3 vHorizonAirspeedVector = {0};
		GetHorizonAirspeedVector (vHorizonAirspeedVector);
		double VertSpeed = vHorizonAirspeedVector.y;
		if (VertSpeed < -3) {
			// we touched ground with more than -3 m/s, sorry dude, time to kill you all : (
			//	for (I = 0;I<Crew.GetCrewTotalNumber ();I++)
			//	{
			//	Crew.SetCrewMemberPulseBySlotNumber (I, 0);	// set cardiac pulse to zero
			//	}
			//	strcpy (SendFocusScreenMessage (), "Oooh no ! Crash - All crew aboard killed");
		}
	}
	//----------------------------------------------------------------------------
	// see function "SendFocusScreenMessage" below for explaination.
	if (GetHandle () == oapiGetFocusObject ()) strcpy (oapiDebugString (), cOurMessage);
	if ((Crew.GetCrewTotalNumber ()) == 1) SetMeshVisibilityMode (iMeshEVA1A, MESHVIS_ALWAYS);
	else SetMeshVisibilityMode (iMeshEVA1A, MESHVIS_NEVER);
	if ((Crew.GetCrewTotalNumber ()) == 2) SetMeshVisibilityMode (iMeshEVA2A, MESHVIS_ALWAYS);
	else SetMeshVisibilityMode (iMeshEVA2A, MESHVIS_NEVER);
	//speedcontrol
	//VESSELSTATUS2 vs ;
	// VECTOR3 vel_local, clipped_vel_global ;
	// double vel_mag ;
	// GetStatusEx ( &vs ) ;
	// Global2Local ( vs.rvel, vel_local ) ;
	//	vel_mag = length ( vel_local ) ;
	//	if (vel_mag > 10 ) {
		// vel_local *= (10/vel_mag) ; // scales velocity clipping in all directions
	// }
	// GlobalRot ( vel_local, clipped_vel_global ) ;
	// vs.rvel = clipped_vel_global ;
	VECTOR3 speedvec; // first we define our vector which will recieve the data
	oapiGetFocusShipAirspeedVector (&speedvec);//then we retrieve its x, y, z
/*	VECTOR3 newspeedvec;
	newspeedvec.x = (speedvec.x*-2);
	newspeedvec.y = (speedvec.y*-2);
	newspeedvec.z = (speedvec.z*-2);
	if (GetAirspeed () > 10) {
		AddForce (newspeedvec, _V (0, 0, 0));
	}/**/
	//double leftturn = GetThrusterLevel (th_yaw_l[0]); 
	//double rightturn = GetThrusterLevel (th_yaw_r[0]); 
	// sprintf (oapiDebugString (), "anim %2.2f anim %2.2f anim %2.2f", i, leftturn, i3 );
	double db = simdt * LIFT_SPEED;
	level5 = GetThrusterGroupLevel (THGROUP_RETRO);
	level4 = GetThrusterGroupLevel (THGROUP_MAIN);
	level8 = GetThrusterGroupLevel (THGROUP_ATT_YAWLEFT);
	level9 = GetThrusterGroupLevel (THGROUP_ATT_YAWRIGHT);
	SetAnimation ( anim_control1a, level9);//yaw RIGHT
	SetAnimation ( anim_control1, level8);//yaw LEFT
	SetAnimation ( anim_control1d, level4);//thrust main
	SetAnimation ( anim_control1e, level5);//ATHRUST RETRO
	SetAnimation ( anim_control2a, level9);//ATT RIGHT
	SetAnimation ( anim_control2, level8);//ATT LEFT
	SetAnimation ( anim_control2d, level4);//thrust main
	SetAnimation ( anim_control2e, level5);//ATHRUST RETRO
	SetAnimation ( anim_control3a, level9);//ATT RIGHT
	SetAnimation ( anim_control3, level8);//ATT LEFT
	SetAnimation ( anim_control3d, level4);//thrust main
	SetAnimation ( anim_control3e, level5);//ATHRUST RETRO
	SetAnimation ( anim_GAUGE, TURN_proc);//DIRECTION GAUGE
	SetAnimation ( anim_GAUGE1, TURN_proc);
	SetAnimation ( anim_GAUGE2, TURN_proc);
	//SUNSCREEN
	if (SUNSCREEN_status >= DOOR_RAISING) {
		double da = simdt * LIFT_SPEED;
		if (SUNSCREEN_status == DOOR_RAISING) {
			if (SUNSCREEN_proc > 0.0) SUNSCREEN_proc = max (0.0, SUNSCREEN_proc-da);
			else SUNSCREEN_status = DOOR_UP;
		} else {
			if (SUNSCREEN_proc < 1.0) SUNSCREEN_proc = min (1.0, SUNSCREEN_proc+da);
			else SUNSCREEN_status = DOOR_DOWN;
		}
		SetAnimation (anim_sscreen16, SUNSCREEN_proc);
		SetAnimation (anim_sscreen15, SUNSCREEN_proc);
		SetAnimation (anim_sscreen14, SUNSCREEN_proc);
		SetAnimation (anim_sscreen13, SUNSCREEN_proc);
		SetAnimation (anim_sscreen12, SUNSCREEN_proc);
		SetAnimation (anim_sscreen11, SUNSCREEN_proc);
		SetAnimation (anim_sscreen10, SUNSCREEN_proc);
		SetAnimation (anim_sscreen9, SUNSCREEN_proc);
		SetAnimation (anim_sscreen8, SUNSCREEN_proc);
		SetAnimation (anim_sscreen7, SUNSCREEN_proc);
		SetAnimation (anim_sscreen6, SUNSCREEN_proc);
		SetAnimation (anim_sscreen5, SUNSCREEN_proc);
		SetAnimation (anim_sscreen4, SUNSCREEN_proc);
		SetAnimation (anim_sscreen3, SUNSCREEN_proc);
		SetAnimation (anim_sscreen2, SUNSCREEN_proc);
	}
	//hatch
	if (DOOR0_status >= DOOR_RAISING) {
		double da = simdt * LIFT_SPEED;
		if (DOOR0_status == DOOR_RAISING) {
			if (DOOR0_proc > 0.0) DOOR0_proc = max (0.0, DOOR0_proc-da);
			else DOOR0_status = DOOR_UP;
		} else {
			if (DOOR0_proc < 1.0) DOOR0_proc = min (1.0, DOOR0_proc+da);
			else DOOR0_status = DOOR_DOWN;
		}
		SetAnimation (anim_adoor0, DOOR0_proc);
	}
/*	SetThrusterMax0 (th_main[0], MAXMAINTH + MAXMAINTH * abs (sin (2*PI*TURN_proc)));
	SetThrusterMax0 (th_main[1], MAXMAINTH + MAXMAINTH * abs (sin (2*PI*TURN_proc)));
	SetThrusterMax0 (th_main[2], MAXMAINTH + MAXMAINTH * abs (sin (2*PI*TURN_proc)));
	SetThrusterMax0 (th_main[3], MAXMAINTH + MAXMAINTH * abs (sin (2*PI*TURN_proc)));
	SetThrusterMax0 (th_retro[0], MAXRETROTH + MAXRETROTH * abs (sin (2*PI*TURN_proc)));
	SetThrusterMax0 (th_retro[1], MAXRETROTH + MAXRETROTH * abs (sin (2*PI*TURN_proc)));
	SetThrusterMax0 (th_retro[2], MAXRETROTH + MAXRETROTH * abs (sin (2*PI*TURN_proc)));
	SetThrusterMax0 (th_retro[3], MAXRETROTH + MAXRETROTH * abs (sin (2*PI*TURN_proc)));/**/
/*	VESSELSTATUS vstat;
	GetStatus (vstat);
	if (vstat.status == 1) {
		SetThrusterGroupLevel (THGROUP_MAIN, 0);
		SetThrusterGroupLevel (THGROUP_RETRO, 0);
		SetThrusterGroupLevel (THGROUP_ATT_YAWLEFT, 0);
		SetThrusterGroupLevel (THGROUP_ATT_YAWRIGHT, 0);
	}/**/
	double vsAirSpeed = GetAirspeed ();
	double thrust = GetThrusterGroupLevel (THGROUP_MAIN) + GetThrusterGroupLevel (THGROUP_RETRO);
	if (thrust == 0) SetSurfaceFrictionCoeff (0.1, 0.75);
	else if (vsAirSpeed > MAXSPEED) SetSurfaceFrictionCoeff (0.4, 0.75);
	else SetSurfaceFrictionCoeff (0.01, 0.75);
	// sprintf (oapiDebugString (), "anim %2.2f anim %2.2f ", TURN_proc, i );
	//joystick
	i3 = GetControlSurfaceLevel (AIRCTRL_RUDDER);
	if (i3 < 0) {
		//(TURN_proc = .125);
		rot += 0.005;
		if (rot > .125) rot = .125;
		SetThrusterGroupLevel (THGROUP_ATT_YAWLEFT, abs (8 * rot));
		SetThrusterGroupLevel (THGROUP_ATT_YAWRIGHT, 0);
	} else if (i3 > 0) {
		//(TURN_proc = .875);
		rot -= 0.005;
		if (rot < -.125) rot = -.125;
		SetThrusterGroupLevel (THGROUP_ATT_YAWLEFT, 0);
		SetThrusterGroupLevel (THGROUP_ATT_YAWRIGHT, abs (8 * rot));
	} else if (i3 == 0) {
		if (abs (rot) <= 0.005) rot = 0;
		else if (rot < 0) rot += 0.005;
		else if (rot > 0) rot -= 0.005;
		SetThrusterGroupLevel (THGROUP_ATT_YAWLEFT, 0);
		SetThrusterGroupLevel (THGROUP_ATT_YAWRIGHT, 0);
	}
	if (TURN_proc > 1) TURN_proc = (TURN_proc-1);
	else if (TURN_proc < 0) TURN_proc = (TURN_proc+1);
	if (TURN_check == TURN_DOWN)TURN_proc = (TURN_proc + db);
	else if (TURN_check == TURN_UP)TURN_proc = (TURN_proc - db);
	else if (TURN_check == TURN_STOP)TURN_proc = TURN_proc;
	double ax14 = TURN_proc - rot;
	if (ax14 > 1) ax14 -= 1;
	else if (ax14 < 0) ax14 += 1;
	double ax36 = TURN_proc + rot;
	if (ax36 > 1) ax36 -= 1;
	else if (ax36 < 0) ax36 += 1;
	SetAnimation (anim_axle1, ax14);
	SetAnimation (anim_axle2, TURN_proc);
	SetAnimation (anim_axle3, ax36);
	SetAnimation (anim_axle4, ax14);
	SetAnimation (anim_axle5, TURN_proc);
	SetAnimation (anim_axle6, ax36);
	double thr2 = GetThrusterLevel (th_yaw_l[0]);
	//Anims 
	//sprintf (oapiDebugString (), "anim %2.2f", i );
	double main_thrust = GetThrusterGroupLevel (THGROUP_MAIN);
	double retro_thrust = GetThrusterGroupLevel (THGROUP_MAIN);
	double sinTurn = sin (2 * PI * TURN_proc);
	double cosTurn = cos (2 * PI * TURN_proc);
	VECTOR3 avel;
	GetAngularVel (avel);
	//wheel rotate
	//sprintf (oapiDebugString (), "anim %2.5f anim %2.2f anim %2.2f anim %2.2f", i, i3, i2, i5 );
	//wrate = (avel.y);
	//sprintf (oapiDebugString (), "anim %5.5f", wrate );
	//if ((GetThrusterLevel (th_main[0]) >0 )|| (GetThrusterLevel (th_main[2])>0)|| (GetThrusterLevel (th_main[3])>0)){
	if ((speedvec.z * cosTurn > 0) || (speedvec.x * sinTurn > 0)) {
		if (i > 1)i = 0;
		if (vsAirSpeed > 11) vsAirSpeed = 11;
		if (vsAirSpeed > 0) {
			if (i > 1)i = 0;
			i = i + (vsAirSpeed /100);
			// if (i3! = 0) i = 1+vsAirSpeed; 
			if (vsAirSpeed == 0) i = 0;
			SetAnimation (anim_wheelright, i);
			SetAnimation (anim_wheelleft, i);
			main_on = 1;
			retro_on = 0;
		}
	}
	//Anim Wheels Backward
	//if (GetThrusterLevel (th_retro[0]) >0 )
	//if ((GetThrusterLevel (th_retro[0]) >0 )|| (GetThrusterLevel (th_main[2])>0)|| (GetThrusterLevel (th_main[3])>0))
	if ((speedvec.z * cosTurn < 0) || (speedvec.x * sinTurn < 0)) {
		if (i < 0)i = 1;
		if (vsAirSpeed > 0) {
			if (vsAirSpeed > 11) vsAirSpeed = 11;
			if (i < 0)i = 1;
			i = i - (vsAirSpeed /100);
			if (vsAirSpeed == 0) i = 1;
			// sprintf (oapiDebugString (), "anim %2.2f", i );
			SetAnimation (anim_wheelright, i);
			SetAnimation (anim_wheelleft, i); 
			main_on = 0;
			retro_on = 1;
		}
	}
	//Anim Wheels Forward and Backward when moving without power
	//Anim Wheels Forward and Backward when moving without power
	if (main_thrust == 0 && retro_thrust == 0 && vsAirSpeed > 0) {
		if (main_on == 0 && retro_on == 1) {
			if (i < 0)i = 1;
			if (vsAirSpeed > 0) {
				if (vsAirSpeed > 11) vsAirSpeed = 11;
				if (i < 0)i = 1;
				i = i - (vsAirSpeed /100);
				if (vsAirSpeed == 0) i = 1;
				//sprintf (oapiDebugString (), "anim %2.2f", i );
				SetAnimation (anim_wheelright, i);
				SetAnimation (anim_wheelleft, i);
			}
		}
		if (main_on == 1 && retro_on == 0) {
			if (i > 1)i = 0;
			if (vsAirSpeed > 0) {
				if (vsAirSpeed > 11) vsAirSpeed = 11;
				if (i > 1)i = 0;
				i = i + (vsAirSpeed /100);
				if (vsAirSpeed == 0) i = 1;
				//sprintf (oapiDebugString (), "anim %2.2f", i );
				SetAnimation (anim_wheelright, i);
				SetAnimation (anim_wheelleft, i);
			}
		} 
	}
	if (main_thrust == 0 && retro_thrust == 0) {
		if (i3 < 0){
			if (i2 < 0)i2 = 0;
			if (i2>1) i2 = 0;
			if (i5 > 1)i5 = 0;
			if (i5 < 0)i5 = 1;
			i5 = i5 - .01;//REVERSE
			i2 = i2 + .01;//FORWARD
			SetAnimation (anim_wheelright, i2);//FORWARD
			SetAnimation (anim_wheelleft, i5);//REVERSE
		}
		if (i3 > 0) {
			//t1 = -1;
			if (i2 < 0)i2 = 0;
			if (i2>1) i2 = 0;
			if (i5 > 1)i5 = 0;
			if (i5 < 0)i5 = 1;
			// if (vsAirSpeed > 55) vsAirSpeed = 55;
			//if (sign == 0) i = i + (vsAirSpeed /300);
			// if (sign == 1) i = i - (vsAirSpeed /300);
			i5 = i5 - .01;//REVERSE
			i2 = i2 + .01;//FORWARD
			// if (i5<0)i5 = 1;
			// if (vsAirSpeed == 0) i = 1;
			// sprintf (oapiDebugString (), "anim %2.2f", i2 );
			SetAnimation (anim_wheelright, i5);//REVERSE
			SetAnimation (anim_wheelleft, i2);//FORWARD
		}
	}
	if (i3 == 0) {
		SetTouchdownPoints (
			_V (-0.05774 * sinTurn, -2.162, -0.05774 * cosTurn),
			_V (0.05 * cosTurn + 0.02887 * sinTurn, -2.162, 0.02887 * cosTurn - 0.05 * sinTurn),
			_V (-0.05 * cosTurn + 0.02887 * sinTurn, -2.162, 0.02887 * cosTurn + 0.05 * sinTurn)
		);
		SetThrusterDir (th_main[0], _V (sinTurn, 0, cosTurn));
		SetThrusterDir (th_retro[0], _V (-sinTurn, 0, -cosTurn));
	}
	SetAttitudeMode (RCS_NONE);
}

void LER2::DefineAnimations (void) {
	//sunscreen
	static UINT sscreenGrp1[1] = {GRP_sscreen16};
	static MGROUP_ROTATE screen16 (iMeshMain, sscreenGrp1, 1, _V (0, .078, -.89), _V (-1, 0, 0), (float) (124.8*RAD)); 
	anim_sscreen16 = CreateAnimation (0);
	AddAnimationComponent (anim_sscreen16, 0, 1, &screen16) ; 
	static UINT sscreenGrp2[1] = {GRP_sscreen15};
	static MGROUP_ROTATE screen15 (iMeshMain, sscreenGrp2, 1, _V (0, .078, -.89), _V (-1, 0, 0), (float) (121.2*RAD)); 
	anim_sscreen15 = CreateAnimation (0);
	AddAnimationComponent (anim_sscreen15, 0, 1, &screen15) ; 
	static UINT sscreenGrp3[1] = {GRP_sscreen14};
	static MGROUP_ROTATE screen14 (iMeshMain, sscreenGrp3, 1, _V (0, .078, -.89), _V (-1, 0, 0), (float) (112.2*RAD)); 
	anim_sscreen14 = CreateAnimation (0);
	AddAnimationComponent (anim_sscreen14, 0, 1, &screen14) ; 
	static UINT sscreenGrp4[1] = {GRP_sscreen13};
	static MGROUP_ROTATE screen13 (iMeshMain, sscreenGrp4, 1, _V (0, .078, -.89), _V (-1, 0, 0), (float) (103.8*RAD)); 
	anim_sscreen13 = CreateAnimation (0);
	AddAnimationComponent (anim_sscreen13, 0, 1, &screen13) ; 
	static UINT sscreenGrp5[1] = {GRP_sscreen12};
	static MGROUP_ROTATE screen12 (iMeshMain, sscreenGrp5, 1, _V (0, .078, -.89), _V (-1, 0, 0), (float) (97.8*RAD)); 
	anim_sscreen12 = CreateAnimation (0);
	AddAnimationComponent (anim_sscreen12, 0, 1, &screen12) ; 
	static UINT sscreenGrp6[1] = {GRP_sscreen11};
	static MGROUP_ROTATE screen11 (iMeshMain, sscreenGrp6, 1, _V (0, .078, -.89), _V (-1, 0, 0), (float) (90*RAD)); 
	anim_sscreen11 = CreateAnimation (0);
	AddAnimationComponent (anim_sscreen11, 0, 1, &screen11) ; 
	static UINT sscreenGrp7[1] = {GRP_sscreen10};
	static MGROUP_ROTATE screen10 (iMeshMain, sscreenGrp7, 1, _V (0, .078, -.89), _V (-1, 0, 0), (float) (79.8*RAD)); 
	anim_sscreen10 = CreateAnimation (0);
	AddAnimationComponent (anim_sscreen10, 0, 1, &screen10) ; 
	static UINT sscreenGrp8[1] = {GRP_sscreen9};
	static MGROUP_ROTATE screen9 (iMeshMain, sscreenGrp8, 1, _V (0, .078, -.89), _V (-1, 0, 0), (float) (67.2*RAD)); 
	anim_sscreen9 = CreateAnimation (0);
	AddAnimationComponent (anim_sscreen9, 0, 1, &screen9) ; 
	static UINT sscreenGrp9[1] = {GRP_sscreen8};
	static MGROUP_ROTATE screen8 (iMeshMain, sscreenGrp9, 1, _V (0, .078, -.89), _V (-1, 0, 0), (float) (58.8*RAD)); 
	anim_sscreen8 = CreateAnimation (0);
	AddAnimationComponent (anim_sscreen8, 0, 1, &screen8) ; 
	static UINT sscreenGrp10[1] = {GRP_sscreen7};
	static MGROUP_ROTATE screen7 (iMeshMain, sscreenGrp10, 1, _V (0, .078, -.89), _V (-1, 0, 0), (float) (49.8*RAD)); 
	anim_sscreen7 = CreateAnimation (0);
	AddAnimationComponent (anim_sscreen7, 0, 1, &screen7) ; 
	static UINT sscreenGrp11[1] = {GRP_sscreen6};//
	static MGROUP_ROTATE screen6 (iMeshMain, sscreenGrp11, 1, _V (0, .078, -.89), _V (-1, 0, 0), (float) (39.6*RAD)); 
	anim_sscreen6 = CreateAnimation (0);
	AddAnimationComponent (anim_sscreen6, 0, 1, &screen6) ; 
	static UINT sscreenGrp12[1] = {GRP_sscreen5};
	static MGROUP_ROTATE screen5 (iMeshMain, sscreenGrp12, 1, _V (0, .078, -.89), _V (-1, 0, 0), (float) (31.2*RAD)); 
	anim_sscreen5 = CreateAnimation (0);
	AddAnimationComponent (anim_sscreen5, 0, 1, &screen5) ; 
	static UINT sscreenGrp13[1] = {GRP_sscreen4};
	static MGROUP_ROTATE screen4 (iMeshMain, sscreenGrp13, 1, _V (0, .078, -.89), _V (-1, 0, 0), (float) (23.4*RAD)); 
	anim_sscreen4 = CreateAnimation (0);
	AddAnimationComponent (anim_sscreen4, 0, 1, &screen4) ; 
	static UINT sscreenGrp14[1] = {GRP_sscreen3};
	static MGROUP_ROTATE screen3 (iMeshMain, sscreenGrp14, 1, _V (0, .078, -.89), _V (-1, 0, 0), (float) (13.8*RAD)); 
	anim_sscreen3 = CreateAnimation (0);
	AddAnimationComponent (anim_sscreen3, 0, 1, &screen3) ; 
	static UINT sscreenGrp15[1] = {GRP_sscreen2};
	static MGROUP_ROTATE screen2 (iMeshMain, sscreenGrp15, 1, _V (0, .078, -.89), _V (-1, 0, 0), (float) (4.8*RAD)); 
	anim_sscreen2 = CreateAnimation (0);
	AddAnimationComponent (anim_sscreen2, 0, 1, &screen2) ; 
	//	static UINT sscreenGrp16[1] = {GRP_sscreen1};
	//static MGROUP_ROTATE screen1 (iMeshMain, sscreenGrp16, 1, _V (0, 0, -.929), _V (-1, 0, 0), (float) (0*RAD)); 
	//anim_sscreen1 = CreateAnimation (0);
	//AddAnimationComponent (anim_sscreen1, 0, 1, &screen1) ; 
	//joystick
	//control1
	static UINT STICKGrp3[1] = {GRP_STICK1};
	static MGROUP_ROTATE control1 (iMeshMain, STICKGrp3, 1, _V (-1.044, .148, 2.204), _V (0, 0, 1), (float) (15*RAD)); 
	anim_control1 = CreateAnimation (0);
	AddAnimationComponent (anim_control1, 0, 1, &control1) ; 
	//control1a
	static UINT STICKGrp4[1] = {GRP_STICK1};
	static MGROUP_ROTATE control1a (iMeshMain, STICKGrp4, 1, _V (-1.044, .148, 2.204), _V (0, 0, 1), (float) (-15*RAD)); 
	anim_control1a = CreateAnimation (0);
	AddAnimationComponent (anim_control1a, 0, 1, &control1a) ; 
	//control1D
	static UINT STICKGrp11[1] = {GRP_STICK1};
	static MGROUP_ROTATE control1d (iMeshMain, STICKGrp11, 1, _V (-1.044, .148, 2.204), _V (1, 0, 0), (float) (15*RAD)); 
	anim_control1d = CreateAnimation (0);
	AddAnimationComponent (anim_control1d, 0, 1, &control1d) ; 
	//control1e
	static UINT STICKGrp12[1] = {GRP_STICK1};
	static MGROUP_ROTATE control1e (iMeshMain, STICKGrp12, 1, _V (-1.044, .148, 2.204), _V (1, 0, 0), (float) (-15*RAD)); 
	anim_control1e = CreateAnimation (0);
	AddAnimationComponent (anim_control1e, 0, 1, &control1e) ; 
	//joystick
	//control2
	static UINT STICKGrp13[1] = {GRP_STICK2};
	static MGROUP_ROTATE control2 (iMeshMain, STICKGrp13, 1, _V (.936, .148, 2.204), _V (0, 0, 1), (float) (15*RAD)); 
	anim_control2 = CreateAnimation (0);
	AddAnimationComponent (anim_control2, 0, 1, &control2) ; 
	//control1a
	static UINT STICKGrp14[1] = {GRP_STICK2};
	static MGROUP_ROTATE control2a (iMeshMain, STICKGrp14, 1, _V (.936, .148, 2.204), _V (0, 0, 1), (float) (-15*RAD)); 
	anim_control2a = CreateAnimation (0);
	AddAnimationComponent (anim_control2a, 0, 1, &control2a) ; 
	//control1D
	static UINT STICKGrp21[1] = {GRP_STICK2};
	static MGROUP_ROTATE control2d (iMeshMain, STICKGrp21, 1, _V (.936, .148, 2.204), _V (1, 0, 0), (float) (15*RAD)); 
	anim_control2d = CreateAnimation (0);
	AddAnimationComponent (anim_control2d, 0, 1, &control2d) ; 
	//control1e
	static UINT STICKGrp22[1] = {GRP_STICK2};
	static MGROUP_ROTATE control2e (iMeshMain, STICKGrp22, 1, _V (.936, .148, 2.204), _V (1, 0, 0), (float) (-15*RAD)); 
	anim_control2e = CreateAnimation (0);
	AddAnimationComponent (anim_control2e, 0, 1, &control2e) ; 
	//STICK3
	static UINT RSTICKGrp13[1] = {GRP_STICK3};
	static MGROUP_ROTATE control3 (iMeshMain, RSTICKGrp13, 1, _V (0, -.23, -2.944), _V (0, 0, 1), (float) (-15*RAD)); 
	anim_control3 = CreateAnimation (0);
	AddAnimationComponent (anim_control3, 0, 1, &control3) ; 
	//control1a
	static UINT RSTICKGrp14[1] = {GRP_STICK3};
	static MGROUP_ROTATE control3a (iMeshMain, RSTICKGrp14, 1, _V (0, -.23, -2.944), _V (0, 0, 1), (float) (15*RAD)); 
	anim_control3a = CreateAnimation (0);
	AddAnimationComponent (anim_control3a, 0, 1, &control3a) ; 
	//control1D
	static UINT RSTICKGrp21[1] = {GRP_STICK3};
	static MGROUP_ROTATE control3d (iMeshMain, RSTICKGrp21, 1, _V (0, -.23, -2.944), _V (1, 0, 0), (float) (-15*RAD)); 
	anim_control3d = CreateAnimation (0);
	AddAnimationComponent (anim_control3d, 0, 1, &control3d) ; 
	//control1e
	static UINT RSTICKGrp22[1] = {GRP_STICK3};
	static MGROUP_ROTATE control3e (iMeshMain, RSTICKGrp22, 1, _V (0, -.23, -2.944), _V (1, 0, 0), (float) (15*RAD)); 
	anim_control3e = CreateAnimation (0);
	AddAnimationComponent (anim_control3e, 0, 1, &control3e) ; 
	//wheels
	ANIMATIONCOMPONENT_HANDLE parent1, parent2, parent3, parent4, parent5, parent6;
	// Wheel4
	static UINT WGrp4[2] = {GRP_WHEEL4A, GRP_WHEEL4B};
	wheel4 = new MGROUP_ROTATE (iMeshMain, WGrp4, 2, _V (-1.529, -1.759, 1.493), _V (1, 0, 0), (float) (360*RAD));
	// wheel4 axle
	static UINT TuGrp4[1] = {GRP_WHEEL4C};
	static MGROUP_ROTATE tu4 (iMeshMain, TuGrp4, 1, _V (-1.5363, -1.759, 1.493), _V (0, 1, 0), (float) (360*RAD));
	// Wheel5
	static UINT WGrp5[2] = {GRP_WHEEL5A, GRP_WHEEL5B};
	wheel5 = new MGROUP_ROTATE (iMeshMain, WGrp5, 2, _V (-1.5363, -1.759, -.569), _V (1, 0, 0), (float) (360*RAD));
	// wheel5 axle
	static UINT TuGrp5[1] = {GRP_WHEEL5C};
	static MGROUP_ROTATE tu5 (iMeshMain, TuGrp5, 1, _V (-1.5363, -1.759, -.569), _V (0, 1, 0), (float) (360*RAD));
	//WHEEL6
	static UINT WGrp6[2] = {GRP_WHEEL6A, GRP_WHEEL6B};
	wheel6 = new MGROUP_ROTATE (iMeshMain, WGrp6, 2, _V (-1.529, -1.759, -2.629), _V (1, 0, 0), (float) (360*RAD));
	// wheel6 axle
	static UINT TuGrp6[1] = {GRP_WHEEL6C};
	static MGROUP_ROTATE tu6 (iMeshMain, TuGrp6, 1, _V (-1.5374, -1.759, -2.629), _V (0, 1, 0), (float) (360*RAD));
	//LEFTWHEEL REVERSE
	//wheel 4 reverse
	static UINT WGrp41[2] = {GRP_WHEEL4A, GRP_WHEEL4B};
	wheel4A = new MGROUP_ROTATE (iMeshMain, WGrp41, 2, _V (-1.529, -1.759, 1.493), _V (1, 0, 0), (float) (-360*RAD));
	// wheel4 axle
	static UINT TuGrp4A[1] = {GRP_WHEEL4C};
	static MGROUP_ROTATE tu4A (iMeshMain, TuGrp4A, 1, _V (-1.5363, -1.759, 1.493), _V (0, 1, 0), (float) (-360*RAD));
	// Wheel5 reverse
	static UINT WGrp51[2] = {GRP_WHEEL5A, GRP_WHEEL5B};
	wheel5A = new MGROUP_ROTATE (iMeshMain, WGrp51, 2, _V (-1.5363, -1.759, -.569), _V (1, 0, 0), (float) (-360*RAD));
	// wheel5 axle
	static UINT TuGrp5A[1] = {GRP_WHEEL5C};
	static MGROUP_ROTATE tu5A (iMeshMain, TuGrp5A, 1, _V (-1.5363, -1.759, -.569), _V (0, 1, 0), (float) (-360*RAD));
	//WHHEL6 reverse
	static UINT WGrp61[2] = {GRP_WHEEL6A, GRP_WHEEL6B};
	wheel6A = new MGROUP_ROTATE (iMeshMain, WGrp61, 2, _V (-1.529, -1.759, -2.629), _V (1, 0, 0), (float) (-360*RAD));
	// wheel6 axlerear Left Wheel Turn
	static UINT TuGrp6A[1] = {GRP_WHEEL6C};
	static MGROUP_ROTATE tu6A (iMeshMain, TuGrp6A, 1, _V (-1.5374, -1.759, -2.629), _V (0, 1, 0), (float) (-360*RAD));
	anim_wheelleft = CreateAnimation (0);
	AddAnimationComponent (anim_wheelleft, 0, 1, wheel4);
	AddAnimationComponent (anim_wheelleft, 0, 1, wheel5);
	AddAnimationComponent (anim_wheelleft, 0, 1, wheel6);
	// anim_wheelleftrev = CreateAnimation (0);
	// AddAnimationComponent (anim_wheelleftrev, 0, 1, wheel4A);
	// AddAnimationComponent (anim_wheelleftrev, 0, 1, wheel5A);
	// AddAnimationComponent (anim_wheelleftrev, 0, 1, wheel6A);
	anim_rudder = CreateAnimation (0.5);
	// anim_wheel = CreateAnimation (0);
	// anim_wheel1 = CreateAnimation (0);
	//sample
	///static UINT gear_groups[2] = {5, 6};
	//static MGROUP_ROTATE gear (0, gear_groups, 2, _V (0, -1.0, 8.5), _V (1, 0, 0), (float) (0.45*PI));
	//static UINT wheel_groups[2] = {10, 11};
	//wheel = new MGROUP_ROTATE (0, wheel_groups, 2, _V (0, -1.0, 6.5), _V (1, 0, 0), (float) (2*PI));
	//anim_gear = CreateAnimation (0.0);
	//parent = AddAnimationComponent (anim_gear, 0, 1, &gear);
	//anim_wheel = CreateAnimation (0.0);
	//AddAnimationComponent (anim_wheel, 0, 1, wheel, parent);
	//RIGHT SIDE
	//Rear Right Wheel
	//wheel1
	static UINT WGrp1[2] = {GRP_WHEEL1A, GRP_WHEEL1B};
	wheel1 = new MGROUP_ROTATE (iMeshMain, WGrp1, 2, _V (1.5374, -1.759, 1.493), _V (1, 0, 0), (float) (360*RAD));
	// wheel1 axle
	static UINT TuGrp1[1] = {GRP_WHEEL1C};
	static MGROUP_ROTATE tu1 (iMeshMain, TuGrp1, 1, _V (1.5374, -1.759, 1.493), _V (0, 1, 0), (float) (360*RAD));
	//WHEEL2
	static UINT WGrp2[2] = {GRP_WHEEL2A, GRP_WHEEL2B};
	wheel2 = new MGROUP_ROTATE (iMeshMain, WGrp2, 2, _V (1.5374, -1.759, -.569), _V (1, 0, 0), (float) (360*RAD));
	// wheel2 axle
	static UINT TuGrp2[1] = {GRP_WHEEL2C};
	static MGROUP_ROTATE tu2 (iMeshMain, TuGrp2, 1, _V (1.5374, -1.759, -.569), _V (0, 1, 0), (float) (360*RAD));
	//wheel3
	static UINT WGrp3[2] = {GRP_WHEEL3A, GRP_WHEEL3B};
	wheel3 = new MGROUP_ROTATE (iMeshMain, WGrp3, 2, _V (1.5374, -1.759, -2.629), _V (1, 0, 0), (float) (360*RAD));
	// wheel3 axle
	static UINT TuGrp3[1] = {GRP_WHEEL3C};
	static MGROUP_ROTATE tu3 (iMeshMain, TuGrp3, 1, _V (1.5374, -1.759, -2.629), _V (0, 1, 0), (float) (360*RAD));
	//wheel1 reverse
	// static UINT WGrp11[2] = {GRP_WHEEL1A, GRP_WHEEL1B};
	// wheel1A = new MGROUP_ROTATE (iMeshMain, WGrp11, 2, _V (1.5374, -1.759, 1.493), _V (1, 0, 0), (float) (-360*RAD));
	// wheel2 reverse
	// static UINT TuGrp1A[1] = {GRP_WHEEL1C};
	// static MGROUP_ROTATE tu1A (iMeshMain, TuGrp1A, 1, _V (1.5374, -1.759, 1.493), _V (0, 1, 0), (float) (-360*RAD));
	//WHEEL2 reverse
	// static UINT WGrp12[2] = {GRP_WHEEL2A, GRP_WHEEL2B};
	// wheel2A = new MGROUP_ROTATE (iMeshMain, WGrp12, 2, _V (1.5374, -1.759, -.569), _V (1, 0, 0), (float) (-360*RAD));
	// wheel2 axle
	// static UINT TuGrp2A[1] = {GRP_WHEEL2C};
	// static MGROUP_ROTATE tu2A (iMeshMain, TuGrp2A, 1, _V (1.5374, -1.759, -.569), _V (0, 1, 0), (float) (-360*RAD));
	//wheel3 reverse
	// static UINT WGrp31[2] = {GRP_WHEEL3A, GRP_WHEEL3B};
	// wheel3A = new MGROUP_ROTATE (iMeshMain, WGrp31, 2, _V (1.5374, -1.759, -2.629), _V (1, 0, 0), (float) (-360*RAD));
	// wheel3 axle
	// static UINT TuGrp3A[1] = {GRP_WHEEL3C};
	// static MGROUP_ROTATE tu3A (iMeshMain, TuGrp3A, 1, _V (1.5374, -1.759, -2.629), _V (0, 1, 0), (float) (-360*RAD));
	anim_wheelright = CreateAnimation (0);
	AddAnimationComponent (anim_wheelright, 0, 1, wheel1);
	AddAnimationComponent (anim_wheelright, 0, 1, wheel2);
	AddAnimationComponent (anim_wheelright, 0, 1, wheel3);
	//anim_wheelrightrev = CreateAnimation (0);
	// AddAnimationComponent (anim_wheelrightrev, 0, 1, wheel1);
	// AddAnimationComponent (anim_wheelrightrev, 0, 1, wheel2);
	// AddAnimationComponent (anim_wheelrightrev, 0, 1, wheel3);
	//axles
	anim_axle1 = CreateAnimation (0.0);
	parent1 = AddAnimationComponent (anim_axle1, 0, 1, &tu1);
	anim_wheel1 = CreateAnimation (0.0);
	AddAnimationComponent (anim_wheel1, 0, 1, wheel1, parent1);
	anim_axle2 = CreateAnimation (0.0);
	parent2 = AddAnimationComponent (anim_axle2, 0, 1, &tu2);
	anim_wheel2 = CreateAnimation (0.0);
	AddAnimationComponent (anim_wheel2, 0, 1, wheel2, parent2);
	anim_axle3 = CreateAnimation (0.0);
	parent3 = AddAnimationComponent (anim_axle3, 0, 1, &tu3);
	anim_wheel3 = CreateAnimation (0.0);
	AddAnimationComponent (anim_wheel3, 0, 1, wheel3, parent3);
	anim_axle4 = CreateAnimation (0.0);
	parent4 = AddAnimationComponent (anim_axle4, 0, 1, &tu4);
	anim_wheel4 = CreateAnimation (0.0);
	AddAnimationComponent (anim_wheel4, 0, 1, wheel4, parent4);
	anim_axle5 = CreateAnimation (0.0);
	parent5 = AddAnimationComponent (anim_axle5, 0, 1, &tu5);
	anim_wheel5 = CreateAnimation (0.0);
	AddAnimationComponent (anim_wheel5, 0, 1, wheel5, parent5);
	anim_axle6 = CreateAnimation (0.0);
	parent6 = AddAnimationComponent (anim_axle6, 0, 1, &tu6);
	anim_wheel6 = CreateAnimation (0.0);
	AddAnimationComponent (anim_wheel6, 0, 1, wheel6, parent6);
	//sample
	///static UINT gear_groups[2] = {5, 6};
	//static MGROUP_ROTATE gear (0, gear_groups, 2, _V (0, -1.0, 8.5), _V (1, 0, 0), (float) (0.45*PI));
	//static UINT wheel_groups[2] = {10, 11};
	//wheel = new MGROUP_ROTATE (0, wheel_groups, 2, _V (0, -1.0, 6.5), _V (1, 0, 0), (float) (2*PI));
	//anim_gear = CreateAnimation (0.0);
	//parent = AddAnimationComponent (anim_gear, 0, 1, &gear);
	//anim_wheel = CreateAnimation (0.0);
	//AddAnimationComponent (anim_wheel, 0, 1, wheel, parent);
	static UINT EGrp72[3] = {GRP_EXTERIOR1, GRP_exthatch, GRP_doorhardware};//sdoor1rightLEFTairlock0
	static MGROUP_ROTATE adoor (iMeshMain, EGrp72, 3, _V (-1.35, 3.2, .044), _V (0, 1, 0), (float) (60*RAD));
	anim_adoor0 = CreateAnimation (0);
	AddAnimationComponent (anim_adoor0, 0, 1, &adoor); 
	//WHEELPOSITION
	static UINT EGrp75[1] = {GRP_WHEELGAUGE1};//sdoor1rightLEFTairlock0
	static MGROUP_ROTATE GAUGE (iMeshMain, EGrp75, 1, _V (-.956, .208, 2.563), _V (0, 0, 1), (float) (-360*RAD));
	anim_GAUGE = CreateAnimation (0);
	AddAnimationComponent (anim_GAUGE, 0, 1, &GAUGE); 
	static UINT EGrp76[1] = {GRP_WHEELGAUGE2};//sdoor1rightLEFTairlock0
	static MGROUP_ROTATE GAUGE1 (iMeshMain, EGrp76, 1, _V (.842, .208, 2.563), _V (0, 0, 1), (float) (-360*RAD));
	anim_GAUGE1 = CreateAnimation (0);
	AddAnimationComponent (anim_GAUGE1, 0, 1, &GAUGE1); 
	static UINT EGrp77[1] = {GRP_WHEELGAUGE3};//sdoor1rightLEFTairlock0
	static MGROUP_ROTATE GAUGE2 (iMeshMain, EGrp77, 1, _V (-.169, -.089, -2.944), _V (0, 0, 1), (float) (360*RAD));
	anim_GAUGE2 = CreateAnimation (0);
	AddAnimationComponent (anim_GAUGE2, 0, 1, &GAUGE2); 
}

DLLCLBK VESSEL *ovcInit (OBJHANDLE hvessel, int flightmodel) {
	return new LER2 (hvessel, flightmodel);
}

DLLCLBK void ovcExit (VESSEL *vessel) {
	if (vessel) delete (LER2*)vessel;
}

// --------------------------------------------------------------
// Keyboard interface handLER2 (buffered key events)
// --------------------------------------------------------------
int LER2::clbkConsumeBufferedKey (DWORD key, bool down, char *kstate) {
	// only process keydown events
	if (!down) return 0; 
	//---------------------------------------------------------------------------
	// Ummu Key "E" perform the EVA of the selected member
	if (key == OAPI_KEY_E) {
		// PERFORM THE EVA
		int Returned = EvaCrewMemberWithStandardSpacesuit (Crew.GetCrewNameBySlotNumber (SelectedUmmuMember));
		switch (Returned) {
		case TRANSFER_TO_DOCKED_SHIP_OK:
			sprintf (SendFocusScreenMessage (), "Transfer to docked ship Ok - %s transfered", Crew.GetLastEvaedCrewName ());SelectedUmmuMember = 0;
			break;
		case EVA_OK:
			sprintf (SendFocusScreenMessage (), "EVA OK - %s left the ship", Crew.GetLastEvaedCrewName ());SelectedUmmuMember = 0;
			break;
		case ERROR_NO_ONE_ON_BOARD:
			strcpy (SendFocusScreenMessage (), "Error, no one on board, unable to EVA");
			break;
		case ERROR_AIRLOCK_CLOSED:
			strcpy (SendFocusScreenMessage (), "Error, airlock is closed, unable to EVA");
			break;
		case ERROR_DOCKED_SHIP_HAVE_AIRLOCK_CLOSED:
			strcpy (SendFocusScreenMessage (), "Error, docked ship's airlock is closed, unable to transfer");
			break;
		case ERROR_DOCKED_SHIP_IS_FULL:
			strcpy (SendFocusScreenMessage (), "Error, docked ship is already full transfer failed");
			break;
		case ERROR_CREW_MEMBER_NOT_FOUND:
			strcpy (SendFocusScreenMessage (), "Error, no crew by this name in ship");
			break;
		case ERROR_DOCKEDSHIP_DONOT_USE_UMMU:
			strcpy (SendFocusScreenMessage (), "Error, docked ship do not use UMmu 1.5, ask author to add it See http://orbiter.dansteph.com");
			break;
		case ERROR_MISC_ERROR_EVAFAILED:
			strcpy (SendFocusScreenMessage (), "Misc error with UMMU See http://orbiter.dansteph.com to install it again");
			break;
		}
		return TRUE;
	}
	//---------------------------------------------------------------------------
	// Ummu Key "1" Select next member This is just internal to the demo
	if (key == OAPI_KEY_1) {
		// we test there is someone aboard
		if (Crew.GetCrewTotalNumber () == 0) {
			strcpy (SendFocusScreenMessage (), "Sorry no one aboard unable to select");	
			return 1;
		}
		// we test that we select existing member
		if (SelectedUmmuMember<Crew.GetCrewTotalNumber ()-1) SelectedUmmuMember++;
		char * Name = Crew.GetCrewNameBySlotNumber (SelectedUmmuMember);
		sprintf (SendFocusScreenMessage (), "Slot %i %s \"%s\" aged %i Selected for EVA or Transfer, please press \"E\" to EVA \"ESC\" to eject", SelectedUmmuMember, Crew.GetCrewMiscIdBySlotNumber (SelectedUmmuMember), Name, Crew.GetCrewAgeBySlotNumber (SelectedUmmuMember));
		return 1;
	}
	//---------------------------------------------------------------------------
	// Ummu Key "2" Select previous member This is just internal to the demo
	if (key == OAPI_KEY_2) {
		// we test there is someone aboard
		if (Crew.GetCrewTotalNumber () == 0) {
			strcpy (SendFocusScreenMessage (), "Sorry no one aboard unable to select");	
			return 1;
		}
		if (SelectedUmmuMember>0) SelectedUmmuMember--;
		char * Name = Crew.GetCrewNameBySlotNumber (SelectedUmmuMember);
		sprintf (SendFocusScreenMessage (), "Slot %i %s \"%s\" aged %i Selected for EVA or Transfer, please press \"E\" to EVA \"ESC\" to eject", SelectedUmmuMember, Crew.GetCrewMiscIdBySlotNumber (SelectedUmmuMember), Name, Crew.GetCrewAgeBySlotNumber (SelectedUmmuMember));
		return 1;
	}
	//---------------------------------------------------------------------------
	// Ummu Key "A" Switch the airlock door on/off
	// Ummu Key "A" Switch the airlock door on/off
	if (key == OAPI_KEY_D) {	// switch state
		Crew.SetAirlockDoorState (!Crew.GetAirlockDoorState ());
		// display state
		if (doorselected == 0) {
			RevertDD0 ();
			if (Crew.GetAirlockDoorState () == TRUE) strcpy (SendFocusScreenMessage (), "Side Airlock is now open");	
			else strcpy (SendFocusScreenMessage (), "Side Airlock is now closed");
		}
		if (doorselected == 1) {
			if (Crew.GetAirlockDoorState () == TRUE) strcpy (SendFocusScreenMessage (), "Right Rear Airlock is now open");	
			else strcpy (SendFocusScreenMessage (), "Right Rear Airlockis now closed");
		}
		if (doorselected == 2) {
			if (Crew.GetAirlockDoorState () == TRUE) strcpy (SendFocusScreenMessage (), "Left Rear Airlock is now open");	
			else strcpy (SendFocusScreenMessage (), "Left Rear Airlock is now closed");
		}
	}
	// change active dock
	if (key == OAPI_KEY_9) {
		if (iActiveDockNumber>0) iActiveDockNumber--;
		sprintf (SendFocusScreenMessage (), "Active dock number changed to: %i", iActiveDockNumber);
		SetUMMUAirlockPos ();
		return 1;
	}
	//---------------------------------------------------------------------------
	// change active dock
	if (key == OAPI_KEY_0) {
		if (iActiveDockNumber<3)
		iActiveDockNumber++;
		sprintf (SendFocusScreenMessage (), "Active dock number changed to: %i", iActiveDockNumber);
		SetUMMUAirlockPos ();
		return 1;
	} 
	//---------------------------------------------------------------------------
	// Eject the guy
	if (key == OAPI_KEY_ESCAPE) {
		if (Crew.EjectCrewMember (Crew.GetCrewNameBySlotNumber (SelectedUmmuMember)) == EVA_OK) sprintf (SendFocusScreenMessage (), "%s EJECTED", Crew.GetLastEvaedCrewName ());SelectedUmmuMember = 0;
		return 1;
	}
	//---------------------------------------------------------------------------
	// Use a different Mesh (type "C" then EVA someone)
	if (key == OAPI_KEY_C) {
		Crew.SetAlternateMeshToUseForEVASpacesuit ("mmu");	// the stock mmu of orbiter located in "meshes/mmu.msh"
		strcpy (SendFocusScreenMessage (), "Mesh changed");
		return 1;
	}
	//---------------------------------------------------------------------------
	// Get some infos Name of ship and total soul aboard
	if (key == OAPI_KEY_S) {
		sprintf (SendFocusScreenMessage (), "%i souls aboard ship %s, %i seats available", Crew.GetCrewTotalNumber (), GetName (), 2-Crew.GetCrewTotalNumber ());
		return 1;
	}
	if (key == OAPI_KEY_N) {
		sprintf (SendFocusScreenMessage (), "");
		return 1;
	}
	if (key == OAPI_KEY_M) {
		RevertSSCREEN ();
		return 1;
	}
	if (key == OAPI_KEY_4) {	// rotate- 
		TURN_proc = 0;
		return 1;
	}
	if (key == OAPI_KEY_G) {
		SelectCockpitView (CAM);
		CAM = CAM + 1;
		if (CAM > 4) CAM = 0;
		return 1;
	}
	return 0;
}

int LER2::clbkConsumeDirectKey (char *kstate) {
	// sprintf (oapiDebugString (), "anim %2.2f", TURN_proc );
	if (KEYDOWN (kstate, OAPI_KEY_7)) { // turns wheels left
		ROTATELEFT ();
		RESETKEY (kstate, OAPI_KEY_7);
	}
	if (KEYDOWN (kstate, OAPI_KEY_8)) { // turns wheel right
		ROTATERIGHT ();
		RESETKEY (kstate, OAPI_KEY_8);
	}
	return 0;
}

// ==================================================================== 
// clbkVisualCreated used to display UMMU initialisation message 
// because oapiDebugString () doesn't work in clbkSetClassCap
// ==================================================================== 
void LER2::clbkVisualCreated (VISHANDLE vis, int refcount) { 
	MainExternalMeshVisual = GetMesh (vis, 0);
	// We warn the user, in the case UMmu isn't installed it's a good idea to send a "oapiDebugString"
	// Orbiter message, otherwise your addon will not be fully operational and user may not notice. 
	// (nobody read doc, you may be the only one, if you read this send me a postcard ;)
	if (UmmuSdKInitReturnCode == -999) strcpy (SendFocusScreenMessage (), "This addon require UMMU addon wich is not installed or outdated. See http://orbiter.dansteph.com to install it");
	else if (UmmuSdKInitReturnCode != 1) strcpy (SendFocusScreenMessage (), "Misc error with UMMU See http://orbiter.dansteph.com to install it again");
	// just for the demo
	else if (UmmuSdKInitReturnCode == 1) strcpy (SendFocusScreenMessage (), "");
}

// ============================================================== 
// Visual destroyed
// ============================================================== 
void LER2::clbkVisualDestroyed (VISHANDLE vis, int refcount) {
	MainExternalMeshVisual = 0;	
}

// ======================================================================== 
// clbkLoadState function of Orbiter - Save scenario when user exit Orbiter 
// ======================================================================== 
void LER2::clbkSaveState (FILEHANDLE scn) {
	// ORBITER, default vessel parameters
	SaveDefaultState (scn);
	// UMmu, this will save all our members of crew in scenario
	// as airlock state depend of your ship (you may have a real animated airlock)
	// it's your responsabilites to save and reload UMmu's airlock state.
	Crew.SaveAllMembersInOrbiterScenarios (scn);
	VESSEL2::clbkSaveState (scn);
	oapiWriteScenario_float (scn, "WHEELROT", TURN_proc * 360);
}

// ======================================================================== 
// clbkLoadStateEx function of Orbiter - Load Orbiter scenario 
// ======================================================================== 
void LER2::clbkLoadStateEx (FILEHANDLE scn, void *status) {
	float wheelrot = 0.0;
	char *line;
	while (oapiReadScenario_nextline (scn, line)) {
		// UMMU, Load all saved member from scenario. 
		// Return TRUE when a "UMMUCREW" line is encountered 
		// FALSE when it must pass this line to you or default parser
		if (Crew.LoadAllMembersFromOrbiterScenario (line) == TRUE) continue;
        if (!strnicmp (line, "WHEELROT", 8)) {
			sscanf (line + 8, "%f", &wheelrot);
        } else {
            ParseScenarioLineEx (line, status);
			// unrecognised option - pass to Orbiter's generic parser
        }
	}
	TURN_proc = wheelrot / 360;
}

//------------------------------------------------------------------------------------------------------------------
// This below is not mandatory for UMmu to run.
//------------------------------------------------------------------------------------------------------------------
// ========================================================== 
// USING TEXT INTERFACE FOR CREW MANAGMENT (not a good idea)
//
// This is the custom functions that send oapiDebugString () Message on screen
// it simply avoid that the message is sent if our ship don't have the focus
// and ensure that message will be reminded when we focus back on ship
// the Orbiter function below clbkFocusChanged is used to erase message when
// we lost focus.
// It's a bad idea to do an UMmu user interface only by using Debug screen message 
// but if you can't do better you may use this system.
///////////////////////////////////////////////////////////////////////////////////////////
char *LER2::SendFocusScreenMessage () {
	return cOurMessage;
}

void LER2::clbkFocusChanged (bool getfocus, OBJHANDLE hNewVessel, OBJHANDLE hOldVessel) {
	// erase message if we focus to another vessel
	if (!getfocus) strcpy (oapiDebugString (), "");
}

// ================================================================ 
// HUD (I put the anim state on the hud because in Poststep
// it forbids ummu messages to be displayed).
// ================================================================ 

// ========================================================================================== 
// USING STANDARD FUNCTION ID, NAME AND SPACESUIT (Good idea)
// **TEST: replace "Crew.EvaCrewMember" with this function in clbkConsumeBufferedKey (key E)**
// ========================================================================================== 
int LER2::EvaCrewMemberWithStandardSpacesuit (char* CrewName) {
	char TempMiscID[5] = {0};
	int I = 0;
	char MeshToUse[26] = {"ummu/ummu"}; // default mesh if not found
	// retriew the crew's MiscID
	strcpy (TempMiscID, Crew.GetCrewMiscIdByName (CrewName));
	// search if we find this ID in the list
	for (I = 0;I<NUMBER_OF_MMU_TYPE;I++) {
		if (strcmp (UmmuMiscID[I], TempMiscID) == 0) {
			// found ! copy the mesh filename to use for EVA
			strcpy (MeshToUse, UmmuMeshUsed[I]);
			break;
		}
	}
	// Set the mesh filename to use
	Crew.SetAlternateMeshToUseForEVASpacesuit (MeshToUse);
	// Eva the crew member with this mesh
	return Crew.EvaCrewMember (CrewName);
}

void LER2::RevertDD0 (void) {
	DOOR0_status = ((DOOR0_status == DOOR_UP || DOOR0_status == DOOR_RAISING) ?
	DOOR_LOWERING : DOOR_RAISING);
}

void LER2::RevertSSCREEN (void) {
	SUNSCREEN_status = ((SUNSCREEN_status == DOOR_UP || SUNSCREEN_status == DOOR_RAISING) ?
	DOOR_LOWERING : DOOR_RAISING);
}

void LER2::ROTATELEFT (void) {
	double simdt = oapiGetSimStep ();
	// sprintf (oapiDebugString (), "anim %2.2f anim %2.2f anim %2.2f", TURN_proc, db, simdt );
	double db = simdt * LIFT_SPEED;
	TURN_proc -= db;
	if (TURN_proc < 0) TURN_proc += 1;
}

void LER2::ROTATERIGHT (void) {
	double simdt = oapiGetSimStep ();
	double db = simdt * LIFT_SPEED;
	 (TURN_proc += db);
	if (TURN_proc > 1) TURN_proc = (TURN_proc-1);
}
