// ==============================================================
//                 ORBITER MODULE: LER2
//                  Part of the ORBITER SDK
//          Copyright (C) 2002-2004 Martin Schweiger
//                   All rights reserved
//
// LER2.cpp
// Control module for LER2 vessel class
//
// Notes:
// This is an example for a "minimal" vessel implementation which
// only overloads the clbkSetClassCaps method to define vessel
// capabilities and otherwise uses the default VESSEL class
// behaviour.
// ==============================================================

//#ifndef __LER2_H
//#define __LER2_H


#include "orbitersdk.h"
#include "UMmuSDK.h"
// ==============================================================
// Some vessel parameters
// ==============================================================
const double MAXSPEED = 2.78; /* (2.78m/s = 10km/h) */
const double MASS = 4000;
const double FUELMASS = 10;
const double ISP = 5e7;
const double MAXMAINTH = 5000;
const double MAXRETROTH = 5000;
const double RCSISP = 5e7;
const double MAXRCSTH = 5000;

// Interface for derived vessel class: QJBStar
// ==========================================================

class LER2: public VESSEL2 {
public:
	LER2 (OBJHANDLE hObj, int fmodel);
	void clbkSaveState (FILEHANDLE scn);
	void clbkSetClassCaps (FILEHANDLE cfg);
	void clbkPostCreation (void);
	int clbkConsumeDirectKey (char *kstate);
	void clbkLoadStateEx (FILEHANDLE scn, void *status);
	int clbkConsumeBufferedKey (DWORD key, bool down, char *kstate);
	void clbkVisualCreated (VISHANDLE vis, int refcount);
	void DefineAnimations(void); 
	void clbkVisualDestroyed (VISHANDLE vis, int refcount);
	void clbkPostStep (double simtt, double simdt, double mjd);
	void clbkFocusChanged (bool getfocus, OBJHANDLE hNewVessel, OBJHANDLE hOldVessel);
	void LER2::SelectCockpitView (int CAM); 
	void SetUMMUAirlockPos (void);
	enum GEARStatus { GEAR_UP, GEAR_DOWN, GEAR_RAISING, GEAR_LOWERING } GEAR_status;
	enum DOORStatus { DOOR_UP, DOOR_DOWN, DOOR_RAISING, DOOR_LOWERING } DOOR0_status, SUNSCREEN_status, DOOR1_status, DOOR3_status, DOOR4_status, DOOR5_status, DOOR6_status;
	enum HATCHStatus { HATCH_UP, HATCH_DOWN, HATCH_RAISING, HATCH_LOWERING } FORKA_status, LP2A_status, FORK_status, LP2_status;

	void RevertGEAR (void);
	void RevertDD0 (void);
	void RevertSSCREEN (void);
	void RevertDD2 (void);
	void RevertDD3 (void);
	void RevertDD5 (void);
	void RevertDD6 (void);
	void RevertLP2 (void);
	// UMMU 1.5 DECLARATION
	UMMUCREWMANAGMENT Crew;
	void clbkSetClassCaps_UMMu (void); // our special SetClassCapfunction added for more readability
	int SelectedUmmuMember; // for the SDK demo, select the member to eva
	int UmmuSdKInitReturnCode; // to display init message to user (in case Ummu isn't installed on Orbiter)

	// our custom oapiDebugString() function (an orbiter function that display debug text at bottom of screen)
	// Our custom function display only message if we have focus. The variables below is used by this function.
	// don't forget if you use such text interface in your vessel that you must initialise the variables
	// in set class cap and do also some code in clbkFocusChanged
	char cOurMessage[1000];
	char *SendFocusScreenMessage (); 

	// If you want to use standard ID function see bottom of UMmuExample.cpp
	int EvaCrewMemberWithStandardSpacesuit(char* CrewName); 

	THRUSTER_HANDLE th_main[1], th_retro[1], th_yaw_l[2], th_yaw_r[2];
	THRUSTER_HANDLE thg_main, thg_retro, thg_yaw_l, thg_yaw_r;
	MGROUP_ROTATE *wheel1, *wheel2, *wheel3, *wheel4, *wheel5, *wheel6, *wheel1A, *wheel2A, *wheel3A, *wheel4A, *wheel5A, *wheel6A;
	enum IGNITION_check { YES_START, NO_START } IGNITION_check;
	enum TURN_check { TURN_UP, TURN_STOP, TURN_DOWN } TURN_check;

private:
	void Steering (); 
	void RevertAIRLOCKOUTER (void);
	void ROTATERIGHT ();
	void ROTATELEFT ();
	void SPINRIGHT ();
	void SPINLEFT ();
	double i3,i2l,i2,var;
	double thrF,lev;
	double thrR;
	double vsAirSpeed;
	UINT anim_GAUGE;
	UINT anim_GAUGE1;
	UINT anim_GAUGE2;

	UINT iMeshEVA1A;
	UINT iMeshEVA2A;
	UINT iMeshMain;
	UINT anim_adoor0;
	UINT anim_radar;
	UINT anim_adoor1A;
	UINT anim_adoor1;
	UINT anim_FORKMAIN;
	UINT anim_tilt;
	UINT anim_steer;
	UINT anim_adoor0A;
	UINT anim_door1A;
	UINT anim_control1;
	UINT anim_control1a;
	UINT anim_control1d;
	UINT anim_control1e;
	UINT anim_control2;
	UINT anim_control2a;
	UINT anim_control2d;
	UINT anim_control2e;
	UINT anim_control3;
	UINT anim_control3a;
	UINT anim_control3d;
	UINT anim_control3e;
	UINT anim_sscreen15;
	UINT anim_sscreen14;
	UINT anim_sscreen13;
	UINT anim_sscreen12;
	UINT anim_sscreen16;
	UINT anim_sscreen11;
	UINT anim_sscreen10;
	UINT anim_sscreen9;
	UINT anim_sscreen8;
	UINT anim_sscreen7;
	UINT anim_sscreen6;
	UINT anim_sscreen5;
	UINT anim_sscreen4;
	UINT anim_sscreen3;
	UINT anim_sscreen2;
	UINT anim_sscreen1;

	double level5, level6, level4, level8, level9, wrate;
	UINT anim_wheel1;
	UINT anim_wheel2;
	UINT anim_wheel3;
	UINT anim_wheel4;
	UINT anim_wheel5;
	UINT anim_wheel6;
	UINT anim_wheel1A;
	UINT anim_wheel2A;
	UINT anim_wheel3A;
	UINT anim_wheel4A;
	UINT anim_wheel5A;
	UINT anim_wheel6A;
	UINT anim_axle1A;
	UINT anim_axle2A;
	UINT anim_axle3A;
	UINT anim_axle4A;
	UINT anim_axle5A;
	UINT anim_axle6A;
	UINT anim_axle1;
	UINT anim_axle2;
	UINT anim_axle3;
	UINT anim_axle4;
	UINT anim_axle5;
	UINT anim_axle6;
	UINT anim_lift;
	UINT anim_door1;
	UINT anim_wheel;
	UINT anim_rudder;
	UINT anim_wheelright;
	UINT anim_wheelleft;
	UINT anim_wheelrightrev;
	UINT anim_wheelleftrev;
	int iActiveDockNumber,doorselected;
	int whichap;
	int CAM;
	int whichone, do_attach, first_time, f8,liftpad;
	int APSel;
	double apd,pit2,pos,main_on,retro_on,SUNSCREEN_proc;
	double TURN_proc, MAX_GRAP_DIST;
	double LIFT_SPEED;
	double radar_speed;
	double light_SPEED;
	double DOOR0_proc;
	double DOOR1_proc;
	double LIFT_proc;
	int JohnSoundID;
	double db,t1;
	double i,i4l,i4,i5;
	double rot;
};

//#endif // !__LER2_H
